# !!!
# WARNING: This file is autogenerated
# Only modify code within MANUAL() sections
# or your changes may be overwritten later!
# !!!

from __future__ import annotations

from stytch.core.response_base import ResponseBase


class WhoamiResponse(ResponseBase):
    project_id: str
    name: str
