# !!!
# WARNING: This file is autogenerated
# Only modify code within MANUAL() sections
# or your changes may be overwritten later!
# !!!

from __future__ import annotations

from stytch.b2b.models.rbac import OrgPolicy
from stytch.core.response_base import ResponseBase


class GetOrgPolicyResponse(ResponseBase):
    """Response type for `Organizations.get_org_policy`.
    Fields:
      - org_policy: The organization-specific RBAC Policy that contains roles defined for this organization. Organization policies supplement the project-level RBAC policy with additional roles that are specific to the organization.
    """  # noqa

    org_policy: OrgPolicy


class SetOrgPolicyResponse(ResponseBase):
    """Response type for `Organizations.set_org_policy`.
    Fields:
      - org_policy: The organization-specific RBAC Policy that contains roles defined for this organization. Organization policies supplement the project-level RBAC policy with additional roles that are specific to the organization.
    """  # noqa

    org_policy: OrgPolicy
