# !!!
# WARNING: This file is autogenerated
# Only modify code within MANUAL() sections
# or your changes may be overwritten later!
# !!!

from __future__ import annotations

from typing import Optional

import pydantic


class MemberOptions(pydantic.BaseModel):
    """
    Fields:
      - mfa_phone_number: The Member's MFA phone number.
      - totp_registration_id: The Member's MFA TOTP registration ID.
    """  # noqa

    mfa_phone_number: str
    totp_registration_id: str


class MfaRequired(pydantic.BaseModel):
    """
    Fields:
      - member_options: Information about the Member's options for completing MFA.
      - secondary_auth_initiated: If null, indicates that no secondary authentication has been initiated. If equal to "sms_otp", indicates that the Member has a phone number, and a one time passcode has been sent to the Member's phone number. No secondary authentication will be initiated during calls to the discovery authenticate or list organizations endpoints, even if the Member has a phone number.
    """  # noqa

    member_options: Optional[MemberOptions] = None
    secondary_auth_initiated: Optional[str] = None
