# !!!
# WARNING: This file is autogenerated
# Only modify code within MANUAL() sections
# or your changes may be overwritten later!
# !!!

from __future__ import annotations

import pydantic


class ScopeResult(pydantic.BaseModel):
    """
    Fields:
      - scope: The name of the scope.
      - description: A human-readable description of the scope, taken from the RBAC Policy.
      - is_grantable: Indicates whether the scope can be granted. Users can only grant scopes if they have the required permissions.
    """  # noqa

    scope: str
    description: str
    is_grantable: bool
