# !!!
# WARNING: This file is autogenerated
# Only modify code within MANUAL() sections
# or your changes may be overwritten later!
# !!!

from __future__ import annotations

from stytch.consumer.models.connected_apps import (
    ConnectedApp,
    ConnectedAppWithNextClientSecret,
)
from stytch.core.response_base import ResponseBase


class RotateCancelResponse(ResponseBase):
    """Response type for `Secrets.rotate_cancel`.
    Fields:
      - connected_app: The Connected App affected by this operation.
    """  # noqa

    connected_app: ConnectedApp


class RotateResponse(ResponseBase):
    """Response type for `Secrets.rotate`.
    Fields:
      - connected_app: The Connected App affected by this operation.
    """  # noqa

    connected_app: ConnectedApp


class RotateStartResponse(ResponseBase):
    """Response type for `Secrets.rotate_start`.
    Fields:
      - connected_app: The Connected App affected by this operation.
    """  # noqa

    connected_app: ConnectedAppWithNextClientSecret
