# !!!
# WARNING: This file is autogenerated
# Only modify code within MANUAL() sections
# or your changes may be overwritten later!
# !!!

from __future__ import annotations

import enum
from typing import List

import pydantic

from stytch.core.response_base import ResponseBase


class ProjectMetricMetricType(str, enum.Enum):
    UNKNOWN = "UNKNOWN"
    USER_COUNT = "USER_COUNT"
    ORGANIZATION_COUNT = "ORGANIZATION_COUNT"
    MEMBER_COUNT = "MEMBER_COUNT"
    M2M_CLIENT_COUNT = "M2M_CLIENT_COUNT"


class ProjectMetric(pydantic.BaseModel):
    metric_type: ProjectMetricMetricType
    count: int


class MetricsResponse(ResponseBase):
    project_id: str
    metrics: List[ProjectMetric]
